@@ -507,8 +507,8 @@ static int find_near_context_cb(void *context_, void *entry,
             cd = gc_distance_between_geoidal_positions
                 (context->position, &scanned_slot->position);            
             break;
-        case ACCURACY_ROMBOID:
-            cd = romboid_distance_between_geoidal_positions
+        case ACCURACY_RHOMBOID:
+            cd = rhomboid_distance_between_geoidal_positions
                 (context->position, &scanned_slot->position);
             break;
         default:
@@ -739,7 +739,7 @@ int find_near(const PanDB * const db,
         matching_rects[0] = rect;        
         nb_zones = 1U;
     } else {
-        nb_zones = find_zones(db, &rect , matching_rects);
+        nb_zones = find_zones(db, &rect, matching_rects);
     }
     assert(nb_zones >= 1U);
     assert(nb_zones <= 4U);
@@ -748,6 +748,7 @@ int find_near(const PanDB * const db,
     if (stack_inspect == NULL) {
         return -1;
     }
+    assert(matching_rect == &matching_rects[0]);
     int ret;
     do {
         ret = find_near_in_zone(matching_rect,
@@ -758,6 +759,7 @@ int find_near(const PanDB * const db,
                                 position,
                                 distance,
                                 limit);
+        matching_rect++;
     } while (ret == 0 && --nb_zones > 0U);
     free_pnt_stack(stack_inspect);
     
@@ -770,6 +772,7 @@ typedef struct FindInRectIntCBContext_ {
     const Rectangle2D *rect;
     SubSlots limit;
     FindInRectCB cb;
+    FindInRectClusterCB cluster_cb;
     void *context_cb;
 } FindInRectIntCBContext;
 
@@ -786,7 +789,7 @@ static int find_in_rect_context_cb(void *context_, void *entry,
     }
     if (context->db->layer_type == LAYER_TYPE_SPHERICAL ||
         context->db->layer_type == LAYER_TYPE_GEOIDAL) {
-        cd = romboid_distance_between_geoidal_positions
+        cd = rhomboid_distance_between_geoidal_positions
             (context->position, &scanned_slot->position);
     } else {
         cd = distance_between_flat_positions(context->db,
@@ -794,42 +797,85 @@ static int find_in_rect_context_cb(void *context_, void *entry,
                                              &scanned_slot->position);
     }
     if (scanned_slot->key_node != NULL) {
+        if (context->limit <= (SubSlots) 0U) {
+            return 1;
+        }
+        context->limit--;
         if (context->cb != NULL) {
             const int ret =
                 context->cb(context->context_cb, scanned_slot, cd);
             if (ret != 0) {
                 return ret;
             }
-            if (context->limit-- <= (SubSlots) 1U) {
-                return 1;
-            }
         }
     } else {
-        printf("=== [NULL KEY] => %.3f/%.3f (%.2f)\n",
-               scanned_slot->position.latitude,
-               scanned_slot->position.longitude,
-               cd);
+        fprintf(stderr, "=== [NULL KEY] => %.3f/%.3f (%.2f)\n",
+                scanned_slot->position.latitude,
+                scanned_slot->position.longitude,
+                cd);
     }
     return 0;
 }
 
-int find_in_rect(const PanDB * const db,
-                 FindInRectCB cb, void * const context_cb,
-                 const Rectangle2D * const rect, const SubSlots limit)
+static int find_in_rect_cluster_context_cb(void *context_, void *entry,
+                                           const size_t sizeof_entry)
 {
-    if (limit <= (SubSlots) 0) {
+    FindInRectIntCBContext *context = context_;
+    if (context->cluster_cb == NULL) {
         return 0;
     }
+    QuadNode *scanned_node = entry;
+    const Rectangle2D * const rect = context->rect;
+    const Position2D center = {
+        .latitude = (rect->edge1.latitude + rect->edge0.latitude) / 2.0,
+        .longitude = (rect->edge1.longitude + rect->edge0.longitude) / 2.0
+    };
+    if (position_is_in_rect(&center, context->rect) == 0) {
+        return 0;
+    }    
+    const Dimension distance =
+        (fabsf(center.latitude - rect->edge0.latitude) +
+            fabsf(center.longitude - rect->edge0.longitude)) / 2.0;
+    Meters radius;
+    if (context->db->layer_type == LAYER_TYPE_SPHERICAL ||
+        context->db->layer_type == LAYER_TYPE_GEOIDAL) {
+        radius = geoidal_distance_to_meters(distance);
+    } else {
+        radius = (Meters) distance;
+    }
+    assert(scanned_node->type == NODE_TYPE_QUAD_NODE);
+    const NbSlots children = scanned_node->sub_slots;
+    (void) sizeof_entry;
+    
+    const int ret =
+        context->cluster_cb(context->context_cb, &center, radius, children);
+    if (ret != 0) {
+        return ret;
+    }
+    if (context->limit-- <= (SubSlots) 1U) {
+        return 1;
+    }
+    return 0;
+}
+
+static int find_in_rect_in_zone(Rectangle2D * const matching_rect,
+                                PntStack *stack_inspect,
+                                const PanDB * const db,
+                                FindInRectCB cb,
+                                FindInRectClusterCB cluster_cb,
+                                void * const context_cb,
+                                const Rectangle2D * const rect,
+                                const SubSlots limit, const Dimension epsilon)
+{
     const Position2D rect_center = {
         .latitude =
         (rect->edge1.latitude + rect->edge0.latitude) / (Dimension) 2.0,
         .longitude =
         (rect->edge1.longitude + rect->edge0.longitude) / (Dimension) 2.0
     };
+    const _Bool cluster = (epsilon > (Dimension) 0.0);
+    SubSlots max_nb_slots_without_clustering = (SubSlots) 0U;
     Rectangle2D scanned_qbounds = db->qbounds;
-    Rectangle2D matching_rects[1];
-    Rectangle2D *matching_rect = &matching_rects[0];    
-    PntStack *stack_inspect;
     const QuadNode *scanned_node;
     Rectangle2D scanned_children_qbounds[4];
     Rectangle2D *scanned_child_qbound;
@@ -837,14 +883,12 @@ int find_in_rect(const PanDB * const db,
     QuadNodeWithBounds *sqnb;
     Node *scanned_node_child;
     unsigned int t;
-
-    *matching_rect = *rect;
-    stack_inspect = new_pnt_stack(DEFAULT_STACK_SIZE_FOR_SEARCHES,
-                                  sizeof qnb);
+    
     scanned_node = &db->root;
     FindInRectIntCBContext context = {
         .db = db,
         .cb = cb,
+        .cluster_cb = cluster_cb,
         .context_cb = context_cb,
         .position = &rect_center,
         .limit = limit
@@ -852,6 +896,9 @@ int find_in_rect(const PanDB * const db,
     for (;;) {
         assert(scanned_node->type == NODE_TYPE_QUAD_NODE);
         get_qrects_from_qbounds(scanned_children_qbounds, &scanned_qbounds);
+        if (cluster != 0) {
+            max_nb_slots_without_clustering = context.limit / (SubSlots) 4U;
+        }
         t = 0U;
         do {
             scanned_node_child = scanned_node->nodes[t];
@@ -860,6 +907,23 @@ int find_in_rect(const PanDB * const db,
                                       matching_rect) == 0) {
                 continue;
             }
+            if (scanned_node_child->bare_node.type == NODE_TYPE_QUAD_NODE &&
+                cluster != 0 &&
+                scanned_node_child->quad_node.sub_slots >
+                max_nb_slots_without_clustering &&
+                fabsf(scanned_child_qbound->edge1.latitude -
+                      scanned_child_qbound->edge0.latitude) < epsilon &&
+                fabsf(scanned_child_qbound->edge1.longitude -
+                      scanned_child_qbound->edge0.longitude) < epsilon) {
+                context.rect = scanned_child_qbound;
+                const int ret = find_in_rect_cluster_context_cb
+                    (&context,
+                        (void *) scanned_node_child, sizeof *scanned_node_child);
+                if (ret != 0) {
+                    return ret;
+                }
+                continue;
+            }
             if (scanned_node_child->bare_node.type == NODE_TYPE_BUCKET_NODE) {
                 const Bucket *bucket = &scanned_node_child->bucket_node.bucket;
                 context.rect = rect;
@@ -882,9 +946,63 @@ int find_in_rect(const PanDB * const db,
         scanned_node = sqnb->quad_node;
         scanned_qbounds = sqnb->qrect;
     }
+    return 0;
+}
+
+int find_in_rect(const PanDB * const db,
+                 FindInRectCB cb, FindInRectClusterCB cluster_cb,
+                 void * const context_cb,
+                 const Rectangle2D * const rect,
+                 const SubSlots limit, const Dimension epsilon)
+{
+    if (limit <= (SubSlots) 0) {
+        return 0;
+    }
+    Rectangle2D matching_rects[4];
+    Rectangle2D *matching_rect = &matching_rects[0];
+    PntStack *stack_inspect;
+    unsigned int nb_zones;
+    
+    if (db->layer_type == LAYER_TYPE_FLAT) {
+        matching_rects[0] = *rect;
+        nb_zones = 1U;
+    } else {
+        Rectangle2D orect = *rect;
+        if (orect.edge0.latitude > orect.edge1.latitude) {
+            orect.edge1.latitude +=
+                db->qbounds.edge1.latitude - db->qbounds.edge0.latitude;
+        }
+        if (orect.edge0.longitude > orect.edge1.longitude) {
+            orect.edge1.longitude +=
+                db->qbounds.edge1.longitude - db->qbounds.edge0.longitude;
+        }        
+        nb_zones = find_zones(db, &orect, matching_rects);
+    }
+    assert(nb_zones >= 1U);
+    assert(nb_zones <= 4U);
+    
+    stack_inspect = new_pnt_stack(DEFAULT_STACK_SIZE_FOR_SEARCHES,
+                                  sizeof(QuadNodeWithBounds));
+    if (stack_inspect == NULL) {
+        return -1;
+    }    
+    assert(matching_rect == &matching_rects[0]);
+    int ret;
+    do {
+        ret = find_in_rect_in_zone(matching_rect,
+                                   stack_inspect,
+                                   db,
+                                   cb,
+                                   cluster_cb,
+                                   context_cb,
+                                   matching_rect,
+                                   limit,
+                                   epsilon);
+        matching_rect++;
+    } while (ret == 0 && --nb_zones > 0U);    
     free_pnt_stack(stack_inspect);
 
-    return 0;
+    return ret;
 }
 
 int init_pan_db(PanDB * const db,
